import { Component, inject } from '@angular/core';
import { Editor } from 'ngx-editor';
import { Place } from 'src/interfaces/interfaces';
import { DataService } from 'src/services/data/data.service';

@Component({
  selector: 'app-journal-places',
  templateUrl: './journal-places.component.html',
  styleUrl: './journal-places.component.scss',
})
export class JournalPlacesComponent {
  shortEditor: Editor = new Editor();
  longEditor: Editor = new Editor();
  toolbar: any = [
    // default value
    ['bold', 'italic'],
    ['bullet_list'],
    [{ heading: ['h3', 'h4', 'h5', 'h6'] }],
  ];

  public isNewPlace = false;
  public isInEditMode = false;
  public currentPlace: Place = {
    name: '',
    shortDescription: '',
    longDescription: '',
  };
  public places: Place[] = [];
  public currentIndex: number = 0;
  private backupIndex: number = -1;
  public showShortDescription: boolean = false;

  private dataService: DataService = inject(DataService);

  ngOnInit(): void {
    this.places = this.dataService.places;

    // if the list is empty, set the currentIndex to -1 to hide the entry-container
    if (this.places.length === 0) {
      this.currentIndex = -1;
    } else {
      this.selectPlace(0);
    }
  }

  // Template functions

  /**
   * Selectes a place from the list indeicated by the index param. It also sets all the needed variables to display the place in read mode.
   * @param index The index of the place to select
   */
  public selectPlace(index: number): void {
    this.currentIndex = index;
    this.currentIndex = index;
    this.currentPlace = this.getPlace();
    this.isNewPlace = false;
    this.isInEditMode = false;
  }

  /**
   * Adds a new place and starts the edit mode.
   */
  public addPlace(): void {
    this.isNewPlace = true;
    this.currentPlace = {
      name: '',
      shortDescription: '',
      longDescription: '',
    };
    this.isNewPlace = true;
    this.isInEditMode = true;
    this.backupIndex = this.currentIndex;
    // Hightlight no place because the placeholder is shown as active
    this.currentIndex = -1;
  }

  public editPlace(): void {
    this.isInEditMode = true;
  }

  /**
   * Saves the current place at the start of the list of places and uploads it to the data service.
   */
  public savePlace(): void {
    if (this.isNewPlace) {
      // Prepend the new JournalEntry
      this.places.unshift(this.currentPlace);
      this.isNewPlace = false;
      this.currentIndex = 0;
    }
    this.isInEditMode = false;
    this.places[this.currentIndex] = this.currentPlace;
    this.uploadPlaces();
  }

  /**
   * Deletes the current place from the list and uploads the new list to the data service.
   */
  public deletePlace(): void {
    this.places.splice(this.currentIndex, 1);
    if (this.places.length === 0) {
      this.currentIndex = -1;
    } else {
      this.currentIndex = Math.max(this.currentIndex - 1, 0);
      this.currentPlace = this.getPlace();
    }
    this.uploadPlaces();
  }

  /**
   * Discards the current place and resets the currentPlace to the last saved version.
   */
  public discardPlace(): void {
    if (this.isNewPlace) {
      this.currentIndex = this.backupIndex;
      this.isNewPlace = false;
    }
    if (this.places.length > 0) {
      // Reset the currentEntry to the last saved version
      this.currentPlace = this.getPlace();
    }
    this.isInEditMode = false;
  }

  /**
   * Saves the current entries in the data service.
   */
  private uploadPlaces(): void {
    console.log('Uploading places in places component');

    this.dataService.places = this.places;
  }

  // Helper functions

  /**
   * Returns a deep copy of the current place.
   */
  private getPlace(): Place {
    return JSON.parse(JSON.stringify(this.places[this.currentIndex]));
  }

  ngOnDestroy(): void {
    this.shortEditor.destroy();
    this.longEditor.destroy();
  }
}
